/* --------------------------------------------------------------------
   Module:  roomwrek.cpp
   Subject: RoomWreck v0.9
   Author:  Golden Spud
   --------------------------------------------------------------------
   Description: Manages roomreq.??? files using following steps:
                1. reads room.dat (C+/65 format) and netid.cit
                2. compares them, gets rid of rooms that exist (keeps
                    a list of deleted rooms, in other words)
                3. reads roomreq file
                4. compares deleted rooms and roomreq file, getting
                    rid of all deleted rooms from roomreq
                5. writes out new roomreq that does not have deleted
                    rooms
   --------------------------------------------------------------------
   History:    (insert new entries at top of list)
   YYmmmDD/nam description
   96feb22/MRS started development
   -------------------------------------------------------------------- */

/* --- Includes ------------------------------------------------------- */
#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include <string.hpp>       // WATCOM C++ dependent
#include <vector.h>
#include <algo.h>
#include <bool.h>

#include "genlib.h"
#include "types.h"

/* --- Defines -------------------------------------------------------- */

/* --- Constants ------------------------------------------------------ */
const char* banner =
    "RoomWreck v0.9 -- roomreq.??? file manager extraordinaire!\n"
    "by Golden Spud @ Golden Spud's Greasehouse.\n";

const word BM1_INUSE =        0x0001;
const word BM1_PUBLIC =       0x0002;
const word BM1_DIRECTORY =    0x0004;
const word BM1_PERMANENT =    0x0008;
const word BM1_GROUPONLY =    0x0010;
const word BM1_READONLY =     0x0020;
const word BM1_DOWNLOADONLY = 0x0040;
const word BM1_SHARED =       0x0080;
const word BM1_MODERATED =    0x0100;
const word BM1_APPLICATION =  0x0200;
const word BM1_BIO =          0x0400;
const word BM1_UPLOADONLY =   0x0800;
const word BM1_PRIVGRP =      0x1000;
const word BM1_ANONYMOUS =    0x2000;
const word BM1_ASKSUBJECT =   0x4000;
const word BM1_GRPMODERATED = 0x8000;

const word BM2_ARCHIVEMSGS =  0x0001;
const word BM2_BOOLGROUP =    0x0002;

const word BM3_AUTOAPP =      0x0001;
const word BM3_NOEXCLUDE =    0x0002;

/* --- Function Objects ----------------------------------------------- */

class strless_i {
    public:
        bool operator()(const string& a, const string& b) {
            return a.lower() < b.lower();
        }
};

/* --- Types ---------------------------------------------------------- */
typedef String string;

struct tRoomDat {
    word Bitmapped_1;
    word Bitmapped_2;
    char Name[31];
    char DirName[64];
    char RoomTell[13];
    char ApplicName[13];
    byte Unused_2;
    word GroupNumber;
    dword Unused_3;
    word PrivGrpNum;
    word Bitmapped_3;
    char Infoline[80];
    char NetID[31];
    char ArchiveFile[13];
    char Dictionary[13];
    char Moderator[31];
    char ExclNetPartner[31];
    byte Unused_5;
    byte BoolGroup[50];
    word UNKNOWN_1;
};

typedef vector<string> stringcontainer;

/* --- Prototypes ----------------------------------------------------- */

/* --- Global Variables ----------------------------------------------- */

/* --- Local Variables ------------------------------------------------ */

/* --- Implementation ------------------------------------------------- */
int main(int argc, char** argv)
{
    cout << banner;
    
    if (argc != 4) {
        cerr << "Command-line format is:\n"
             << "  " << argv[0] << " room.dat netid.cit roomreq.abc\n"
             << "where room.dat is the path and filename of your room.dat,\n"
             << "      netid.cit is the path and filename of your netid.cit,\n"
             << "  and roomreq.abc is the path and filename of the roomreq "
             << "to process.\n";
        return 1;
    }

    string fnRoomDat = argv[1];
    string fnNetidCit = argv[2];
    string fnRoomreq = argv[3];

    // ######################## 1a. READ ROOM.DAT ##########################

    cout << "Reading in room.dat...";

    stringcontainer vRoomDat;
    ifstream fRoomDat(fnRoomDat, ios::in | ios::binary);
    if (!fRoomDat) {
        cerr << fnRoomDat << " cannot be opened.\n";
        return 1;
    }

    dword reclen;
    fRoomDat.read((char*)&reclen, sizeof (reclen));
    if (reclen != sizeof (tRoomDat)) {
        cerr << "room.dat is in an invalid format.\n"
             << reclen << "=reclen, "
             << sizeof (tRoomDat) << "=sizeof(tRoomDat)\n";
        return 1;
    }

    tRoomDat RoomInfo;
    for (fRoomDat.read((char*)&RoomInfo, sizeof (RoomInfo)); fRoomDat;
         fRoomDat.read((char*)&RoomInfo, sizeof (RoomInfo)))
    {
        if (RoomInfo.Bitmapped_1 & BM1_INUSE)
            if (RoomInfo.Bitmapped_1 & BM1_SHARED)
                vRoomDat.push_back(RoomInfo.NetID);
        #ifndef NDEBUG
            cout << "\n" << RoomInfo.NetID;
        #endif
    }
    cout << " " << vRoomDat.size() << " net rooms.\n";
    fRoomDat.close();

    cout << "Sorting vRoomDat...\n";
    sort(vRoomDat.begin(), vRoomDat.end(), strless_i());

    // ######################## 1b. READ NETID.CIT #########################

    cout << "Reading in netid.cit...";

    stringcontainer vNetidCit;
    ifstream fNetidCit(fnNetidCit);
    if (!fNetidCit) {
        cerr << fnNetidCit << " cannot be opened.\n";
        return 1;
    }

    static char scratch[128];
    for (fNetidCit.getline(scratch, 127); fNetidCit;
         fNetidCit.getline(scratch, 127))
    {
        if (scratch[strlen(scratch) - 1] == '\n')
            scratch[strlen(scratch) - 1] = 0;
            
        vector<string> words = parse_it(scratch);

        if (words.size() != 3 || words[0].lower() != "#netid")
            continue;

        vNetidCit.push_back(words[1]);

        #ifndef NDEBUG
            cout << "\n" << words[1];
        #endif
    }
    cout << " " << vNetidCit.size() << " net ID's\n";
    fNetidCit.close();

    cout << "Sorting vNetidCit...\n";
    sort(vNetidCit.begin(), vNetidCit.end(), strless_i());

    // #################### 2. keep only deleted rooms ##############

    cout << "Comparing vRoomDat and vNetidCit...";

    stringcontainer vDeleted;
    set_difference(vNetidCit.begin(), vNetidCit.end(),
                   vRoomDat.begin(), vRoomDat.end(),
                   back_inserter(vDeleted),
                   strless_i());

    cout << " " << vDeleted.size() << " deleted rooms.\n";

    #ifndef NDEBUG
        for (stringcontainer::iterator vDi = vDeleted.begin();
             vDi != vDeleted.end(); ++vDi)
        {
            cout << "\n" << *vDi;
        }
    #endif

    // #################### 3. read roomreq file ####################

    cout << "Reading in room request " << fnRoomreq << "...";

    stringcontainer vRoomreq;
    ifstream fRoomreq(fnRoomreq);
    if (!fRoomreq) {
        cerr << fnRoomreq << " cannot be opened.\n";
        return 1;
    }

    // static char scratch[128];
    for (fRoomreq.getline(scratch, 127); fRoomreq;
         fRoomreq.getline(scratch, 127))
    {
        // this first strlen compare prevents scratch[-1] below
        if (strlen(scratch) == 0)
            continue;
        
        if (scratch[strlen(scratch) - 1] == '\n')
            scratch[strlen(scratch) - 1] = 0;

        if (strlen(scratch) == 0)
            continue;

        vRoomreq.push_back(scratch);

        #ifndef NDEBUG
            cout << "\n" << scratch;
        #endif
    }
    cout << " " << vRoomreq.size() << " room requests.\n";
    fRoomreq.close();

    cout << "Sorting vRoomreq...\n";
    sort(vRoomreq.begin(), vRoomreq.end(), strless_i());

    // ########### 4. Compare deleted rooms and roomreq file ################

    cout << "Comparing vRoomreq and vDeleted...";

    stringcontainer vNotDeleted;
    set_difference(vRoomreq.begin(), vRoomreq.end(),
                   vDeleted.begin(), vDeleted.end(),
                   back_inserter(vNotDeleted),
                   strless_i());

    cout << " " << vNotDeleted.size() << " rooms left.\n";

    #ifndef NDEBUG
        for (stringcontainer::iterator vNDi = vNotDeleted.begin();
             vNDi != vNotDeleted.end(); ++vNDi)
        {
            cout << "\n" << *vNDi;
        }
    #endif

    // ############# 5. Write out new roomreq file #################

    cout << "Writing new room request " << fnRoomreq << "...";

    ofstream oRoomreq(fnRoomreq, ios::out | ios::truncate);
    if (!oRoomreq) {
        cerr << fnRoomreq << " cannot be opened.\n";
        return 1;
    }

    for (stringcontainer::iterator i = vNotDeleted.begin();
         i < vNotDeleted.end(); ++i)
    {
        oRoomreq << "\n" << *i;
    }
    cout << "\n";
    oRoomreq.close();

    // ################# DONE ################

    cout << "Done with operation!\n";

    return 0;
}

/* --- End of File ---------------------------------------------------- */
